package service

import (
	"bytes"
	"crypto"
	"crypto/rsa"
	"crypto/sha256"
	"crypto/tls"
	"crypto/x509"
	"encoding/base64"
	"encoding/json"
	"errors"
	"f5"
	"fmt"
	"io"
	"main/mt"
	"net/http"
	"q5"
	"time"
)

type hwpay struct {
	accessTokens q5.ConcurrentMap[int64, TokenInfo] //[gameid, TokenInfo]
	refreshflag  bool
	client       http.Client
}

type HwVerifyOrderRsp struct {
	RspCode    string `json:"responseCode"` //"0":OK;其它，错误
	RspMsg     string `json:"responseMessage"`
	PTokenData string `json:"purchaseTokenData"`  //支付信息
	Sig        string `json:"dataSignature"`      //签名
	SigAlg     string `json:"signatureAlgorithm"` //签名算法
}

type InappPurchaseDetails struct {
	AppId        int64  `json:"applicationId"`    //应用id
	AutoRenew    bool   `json:"autoRenewing"`     //是否自动更新，非订阅型始终为false
	OrderId      string `json:"orderId"`          //订单id
	Kind         int64  `json:"kind"`             //商品类别
	PKGName      string `json:"packageName"`      //包名
	ProdId       string `json:"productId"`        //商品id
	ProdName     string `json:"productName"`      //商品名
	PurTime      int64  `json:"purchaseTime"`     //购买时间
	State        int64  `json:"purchaseState"`    //订单状态 0 已购买
	PayLoad      string `json:"developerPayload"` //保留信息，约定为游戏内orderid
	ConsumeState int64  `json:"consumptionState"` //消耗状态 0 未消耗 1 已消耗
	Price        int64  `json:"price"`            //商品价格 分
}

type HwpayNotifyObj struct {
	V     string `json:"version"`       //"0":OK;其它，错误
	ET    string `json:"eventType"`     //"ORDER" "SUBSCRIPTION"
	Time  int64  `json:"notifyTime"`    //ms
	AppId string `json:"applicationId"` //appid
}

type HwpayOrderNotifyObj struct {
	V    string `json:"version"`          //"0":OK;其它，错误
	Type int64  `json:"notificationType"` //1 支付成功 2 退款成功
	PT   string `json:"purchaseToken"`    //"ORDER" "SUBSCRIPTION"
	PId  string `json:"productId"`        //hw productid
}

func (hp *hwpay) init() {
	hp.accessTokens = q5.ConcurrentMap[int64, TokenInfo]{}

	mt.Table.Hwconfig.Traverse(func(w *mt.Hwconfig) bool {
		hp.accessTokens.Store(w.GetGameid(), TokenInfo{})
		return true
	})

	hp.refreshflag = true
	go hp.checkAccessToken()

	hp.client = http.Client{
		Timeout: time.Second * 5,
		Transport: &http.Transport{
			TLSClientConfig: &tls.Config{
				CipherSuites: []uint16{
					tls.TLS_AES_128_GCM_SHA256,
					tls.TLS_AES_256_GCM_SHA384,
					tls.TLS_CHACHA20_POLY1305_SHA256,
					tls.TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384,
					tls.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256,
					tls.TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384,
					tls.TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256,
				},
				MinVersion: tls.VersionTLS12,
				MaxVersion: tls.VersionTLS13,
			},
		},
	}
}

func (hp *hwpay) unInit() {
}

func (hp *hwpay) freshAccessToken(gameid int64) (token string) {
	cfg := mt.Table.Hwconfig.GetById(gameid)
	params := map[string]string{
		"grant_type":    "client_credentials",
		"client_id":     cfg.GetClientid(),
		"client_secret": cfg.GetClientsecret(),
	}
	url := mt.Table.Config.GetHwAtUrl()
	f5.GetHttpCliMgr().SendGoStyleRequest(
		url,
		params,
		func(hcr f5.HttpCliResponse) {
			if hcr.GetErr() != nil {
				return
			}

			rspObj := struct {
				Token   string `json:"access_token"`
				Expire  int64  `json:"expires_in"`
				ErrCode int64  `json:"error"`
				ErrMsg  string `json:"error_description"`
			}{}

			f5.GetSysLog().Debug("get hw access token rsp:%s", hcr.GetRawData())
			if json.Unmarshal([]byte(hcr.GetRawData()), &rspObj) != nil {
				return
			}

			if rspObj.ErrCode == 0 {
				tokenitem, _ := hp.accessTokens.Load(gameid)
				tokenitem.GameId = gameid
				tokenitem.Token = rspObj.Token
				tokenitem.Expire = rspObj.Expire + f5.GetApp().GetRealSeconds()
				hp.accessTokens.Store(gameid, *tokenitem)
			}

		})

	return token
}

func (hp *hwpay) getAccessToken(gameid int64) (token string) {
	cfg, exist := hp.accessTokens.Load(gameid)
	nowTime := f5.GetApp().GetRealSeconds()
	if exist {
		if cfg.Expire > nowTime {
			if cfg.Expire < nowTime+3 && !hp.refreshflag {
				hp.refreshflag = true
			}
			return cfg.Token
		}

		if !hp.refreshflag {
			hp.refreshflag = true
		}
		return ""
	}
	return ""
}

// return 0 unpaid, 1 paid, 2 consumed
func (hp *hwpay) QueryOrder(gameid int64, purchaseToken string, product_id string, orderid *string) (state int32, sporderid string, orderprice int64) {
	bodyMap := map[string]string{"purchaseToken": purchaseToken, "productId": product_id}

	hosts := mt.Table.Config.GetHwOrderHosts()
	queryuri := "/applications/purchases/tokens/verify"
	cfg := mt.Table.Hwconfig.GetById(gameid)
	if cfg == nil {
		return
	}

	for _, hwhost := range hosts {
		url := "https://" + hwhost + queryuri
		bodybytes, err := hp.sendRequest(gameid, url, bodyMap)
		if err != nil {
			continue
		}

		rsp := HwVerifyOrderRsp{}
		f5.GetSysLog().Debug("get hw verify order rsp:%s", bodybytes)
		if json.Unmarshal(bodybytes, &rsp) != nil {
			return
		}

		if rsp.RspCode != "0" {
			return
		}

		if hp.verifyRsaSign(rsp.PTokenData, rsp.Sig, cfg.GetPublickey()) != nil {
			return
		}

		purdetails := InappPurchaseDetails{}
		if json.Unmarshal([]byte(rsp.PTokenData), &purdetails) != nil {
			return
		}

		if q5.SafeToString(purdetails.AppId) != cfg.GetClientid() {
			return
		}

		if orderid == nil || (len(*orderid) > 0 && purdetails.PayLoad != *orderid) {
			return
		}

		if purdetails.ProdId != product_id {
			return
		}

		if purdetails.State != 0 {
			return
		}

		if purdetails.ConsumeState != 0 {
			state = 2
			return
		}

		sporderid = purdetails.OrderId
		orderprice = purdetails.Price
		if len(*orderid) > 0 {
			// confirm order
			url = "https://" + hwhost + "/applications/v2/purchases/confirm"
			confirmbytes, err := hp.sendRequest(gameid, url, bodyMap)
			if err != nil {
				return
			}

			f5.GetSysLog().Debug("get hw confirm order rsp:%s", confirmbytes)
			confirmrsp := struct {
				RspCode string `json:"responseCode"` //"0":OK;其它，错误
				RspMsg  string `json:"responseMessage"`
			}{}
			if json.Unmarshal(confirmbytes, &confirmrsp) != nil {
				return
			}

			if confirmrsp.RspCode == "0" {
				state = 1
			}
		} else {
			*orderid = purdetails.PayLoad
			state = 1
		}

		return
	}

	return
}

// call queryorder first and its return is 1
func (hp *hwpay) ConfirmOrder(gameid int64, purchaseToken string, product_id string) (ret bool) {
	bodyMap := map[string]string{"purchaseToken": purchaseToken, "productId": product_id}

	hosts := mt.Table.Config.GetHwOrderHosts()
	queryuri := "/applications/v2/purchases/confirm"
	cfg := mt.Table.Hwconfig.GetById(gameid)
	if cfg == nil {
		return
	}

	for _, hwhost := range hosts {
		url := "https://" + hwhost + queryuri
		confirmbytes, err := hp.sendRequest(gameid, url, bodyMap)
		if err != nil {
			continue
		}

		f5.GetSysLog().Debug("get hw confirm order rsp:%s", confirmbytes)
		confirmrsp := struct {
			RspCode string `json:"responseCode"` //"0":OK;其它，错误
			RspMsg  string `json:"responseMessage"`
		}{}
		if json.Unmarshal(confirmbytes, &confirmrsp) != nil {
			return
		}

		return confirmrsp.RspCode == "0"
	}

	return
}

func (hp *hwpay) GetAccessTokenList(data *[]TokenInfo) {
	hp.accessTokens.Range(func(key int64, value TokenInfo) bool {
		*data = append(*data, value)
		return true
	})
}

func (hp *hwpay) checkAccessToken() {
	for {
		time.Sleep(time.Second)

		// if f5.IsOnlineEnv() {
		gamesGoods.Range(func(gameid int64, value map[int64]GoodsInfo) bool {
			cfg, ok := hp.accessTokens.Load(gameid)
			if !ok {
				return true
			}
			if hp.refreshflag || (cfg != nil && cfg.Expire <= f5.GetApp().GetRealSeconds()) {
				hp.freshAccessToken(gameid)
			}
			return true
		})
		// } else {
		// 	if !hp.refreshflag {
		// 		hp.accessTokens.Range(func(key int64, value TokenInfo) bool {
		// 			if value.Expire <= f5.GetApp().GetRealSeconds() {
		// 				hp.refreshflag = true
		// 				return false
		// 			}
		// 			return true
		// 		})

		// 		if !hp.refreshflag {
		// 			continue
		// 		}
		// 	}
		// 	url := "https://payservice.kingsome.cn/api/service/refresh"

		// 	nowtimestr := q5.SafeToString(f5.GetApp().GetRealSeconds())
		// 	params := map[string]string{
		// 		"type": "1",
		// 		"time": nowtimestr,
		// 		"sign": Wxpay.GenSHA256Signature(nowtimestr+constant.GLOBAL_SALT, constant.GLOBAL_SALT),
		// 	}
		// 	f5.GetHttpCliMgr().SendGoStyleRequest(
		// 		url,
		// 		params,
		// 		func(hcr f5.HttpCliResponse) {
		// 			if hcr.GetErr() != nil {
		// 				return
		// 			}

		// 			rspObj := struct {
		// 				Data    []TokenInfo `json:"data"`
		// 				ErrCode int64       `json:"errcode"`
		// 				ErrMsg  string      `json:"errmsg"`
		// 			}{}

		// 			f5.GetSysLog().Debug("hp get online payservice rsp:%s", hcr.GetRawData())
		// 			if json.Unmarshal([]byte(hcr.GetRawData()), &rspObj) != nil {
		// 				return
		// 			}

		// 			if rspObj.ErrCode == 0 {
		// 				for _, dataitem := range rspObj.Data {
		// 					hp.accessTokens.Store(dataitem.GameId, dataitem)
		// 				}
		// 			}
		// 		})
		// }

		hp.refreshflag = false
	}
}

func (hp *hwpay) buildAuthorization(gameid int64) (string, error) {
	token := hp.getAccessToken(gameid)
	if token == "" {
		return "", errors.New("")
	}
	oriString := fmt.Sprintf("APPAT:%s", token)
	var authString = base64.StdEncoding.EncodeToString([]byte(oriString))
	var authHeaderString = fmt.Sprintf("Basic %s", authString)
	return authHeaderString, nil
}

func (hp *hwpay) sendRequest(gameid int64, url string, bodyMap map[string]string) ([]byte, error) {
	authHeader, err := hp.buildAuthorization(gameid)
	if err != nil {
		return nil, err
	}

	bodyString, err := json.Marshal(bodyMap)
	if err != nil {
		return nil, err
	}

	req, err := http.NewRequest("POST", url, bytes.NewReader(bodyString))
	if err != nil {
		return nil, err
	}
	req.Header.Set("Content-Type", "application/json; charset=UTF-8")
	req.Header.Set("Authorization", authHeader)
	response, err := hp.client.Do(req)
	if err != nil {
		return nil, err
	}
	defer response.Body.Close()
	bodyBytes, err := io.ReadAll(response.Body)
	if err != nil {
		return nil, err
	}
	return bodyBytes, nil
}

func (hp *hwpay) verifyRsaSign(content string, sign string, publicKey string) error {
	publicKeyByte, err := base64.StdEncoding.DecodeString(publicKey)
	if err != nil {
		return err
	}
	pub, err := x509.ParsePKIXPublicKey(publicKeyByte)
	if err != nil {
		return err
	}
	hashed := sha256.Sum256([]byte(content))
	signature, err := base64.StdEncoding.DecodeString(sign)
	if err != nil {
		return err
	}
	return rsa.VerifyPKCS1v15(pub.(*rsa.PublicKey), crypto.SHA256, hashed[:], signature)
}
