package service

import (
	"bytes"
	"context"
	"crypto"
	"crypto/rsa"
	"crypto/sha256"
	"encoding/base64"
	"encoding/json"
	"f5"
	"fmt"
	"io"
	"main/mt"
	"mime/multipart"
	"os"
	"q5"
	"time"

	"github.com/wechatpay-apiv3/wechatpay-go/core"
	"github.com/wechatpay-apiv3/wechatpay-go/core/option"
	"github.com/wechatpay-apiv3/wechatpay-go/services/payments/jsapi"
	"github.com/wechatpay-apiv3/wechatpay-go/utils"
)

func (wp *wxpay) initMch() {
	var (
		mchID                      string = mt.Table.Config.GetWxMerchantId()    // 商户号
		mchCertificateSerialNumber string = mt.Table.Config.GetWxCertificateSn() // 商户证书序列号
		// mchAPIv3Key                string = mt.Table.Config.GetWxMerchantApiKey() // 商户APIv3密钥
		mchPubKeyId string = mt.Table.Config.GetWxMerchantPublicKeyId()
	)

	// 使用 utils 提供的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
	mchPrivateKey, err := utils.LoadPrivateKeyWithPath("../res/kingsome_key.pem")
	if err != nil {
		f5.GetSysLog().Alert("load merchant private key error")
	}
	mchPublicKey, err := utils.LoadPublicKeyWithPath("../res/kingsome_pub_key.pem")
	if err != nil {
		f5.GetSysLog().Alert("load merchant public key error")
	}

	wp.ctx = context.Background()
	// 使用商户私钥等初始化 client，并使它具有自动定时获取微信支付平台证书的能力
	opts := []core.ClientOption{
		option.WithWechatPayPublicKeyAuthCipher(mchID, mchCertificateSerialNumber, mchPrivateKey, mchPubKeyId, mchPublicKey),
	}
	wp.client, err = core.NewClient(wp.ctx, opts...)
	if err != nil {
		f5.GetSysLog().Alert("new wechat pay client err:%s", err)
	} else {
		wp.mchpubkey = mchPublicKey
	}

	go wp.checkGameMediaId()

	wp.payhtmlstr, err = f5.ReadJsonFile("../config/payhtml.template")

	if err != nil {
		f5.GetSysLog().Error("%s", err.Error())
	}
}

func (wp *wxpay) checkGameMediaId() {
	wp.mediaInfo = q5.ConcurrentMap[int64, mediaidInfo]{}
	gamecout := 0
	mt.Table.Wxconfig.Traverse(func(w *mt.Wxconfig) bool {
		gamecout++
		return true
	})

	for {
		nowtime := f5.GetApp().GetRealSeconds()
		mt.Table.Wxconfig.Traverse(func(w *mt.Wxconfig) bool {
			info, _ := wp.mediaInfo.Load(w.GetGameid())
			if info == nil {
				info = &mediaidInfo{}
			}

			if q5.GetDaySeconds(info.time, 8) == q5.GetDaySeconds(nowtime, 8) && len(info.mediaId) > 0 {
				return true
			}

			filename := q5.SafeToString(w.GetGameid()) + ".jpg"
			fullfilename := fmt.Sprintf("../res/gamepics/%s", filename)

			body := &bytes.Buffer{}
			writer := multipart.NewWriter(body)
			accesstoken, ok := wp.accessTokens.Load(w.GetGameid())
			if !ok || len(accesstoken.Token) == 0 {
				return true
			}

			file, err := os.Open(fullfilename)
			if err != nil {
				f5.GetSysLog().Error("file error1:%s", filename)
				return true
			}
			defer file.Close()

			part, err := writer.CreateFormFile("media", fullfilename)
			if err != nil {
				f5.GetSysLog().Debug("createFormFile err:%v", err.Error())
				return true
			}
			_, err = io.Copy(part, file)
			if err != nil {
				f5.GetSysLog().Debug("io.copy err:%v", err.Error())
				return true
			}
			writer.Close()

			desurl := "https://api.weixin.qq.com/cgi-bin/media/upload"
			urls := mt.Table.Config.GetWxApiHosts()
			if len(urls) > 0 {
				desurl = fmt.Sprintf("https://%s/cgi-bin/media/upload", urls[0])
			}
			f5.GetHttpCliMgr().SendGoStylePost(
				desurl,
				map[string]string{
					"access_token": accesstoken.Token,
					"type":         "image",
				},
				writer.FormDataContentType(),
				body.String(),
				func(hcr f5.HttpCliResponse) {
					if hcr.GetErr() != nil {
						f5.GetSysLog().Debug("upload err:%s", hcr.GetErr().Error())
						return
					}

					f5.GetSysLog().Debug("upload rsp:%s", hcr.GetRawData())
					rspObj := struct {
						Type    string `json:"type"`
						MediaId string `json:"media_id"`
						Time    int64  `json:"created_at"`
					}{}

					if json.Unmarshal([]byte(hcr.GetRawData()), &rspObj) != nil || len(rspObj.MediaId) == 0 {
						return
					}

					info.mediaId = rspObj.MediaId
					info.time = nowtime
					wp.mediaInfo.Store(w.GetGameid(), *info)
					f5.GetSysLog().Debug("upload refresh media ok:%s, %s", filename, rspObj.MediaId)
				})

			return true
		})

		if wp.mediaInfo.GetSize() != gamecout {
			time.Sleep(time.Second)
		} else {
			time.Sleep(time.Minute)
		}
	}
}

func (wp *wxpay) GenThumburl(gameid int64) (thumburl string) {
	token := wp.getAccessToken(gameid)
	pmediaid, ok := wp.mediaInfo.Load(gameid)
	if ok {
		urls := mt.Table.Config.GetWxApiHosts()
		if len(urls) > 0 {
			thumburl = fmt.Sprintf("https://%s/cgi-bin/media/get?access_token=%s&type=image&media_id=%s", urls[0], token, *pmediaid)
		}

		f5.GetSysLog().Debug("thumburl:%s", thumburl)
	}

	return
}

func (wp *wxpay) GenSendWxMsgUrl(gameid int64) (msgurl string) {
	token := wp.getAccessToken(gameid)
	urls := mt.Table.Config.GetWxApiHosts()
	if len(urls) > 0 {
		msgurl = fmt.Sprintf("https://%s/cgi-bin/message/custom/send?access_token=%s", urls[0], token)
	}

	return
}

func (wp *wxpay) GetPrepayInfoStr(openid string, gameid int64, userip string, orderid string, goodsid int64) (rspstr string) {
	svc := jsapi.JsapiApiService{Client: wp.client}
	// 得到prepay_id，以及调起支付所需的参数和签名
	cfg := mt.Table.Wxconfig.GetById(gameid)
	goods, ok := gamesGoods.Load(gameid)
	if !ok {
		return
	}

	info, ok := (*goods)[goodsid]
	if !ok {
		return
	}

	resp, result, err := svc.PrepayWithRequestPayment(wp.ctx,
		jsapi.PrepayRequest{
			Appid:       core.String(cfg.GetAppid()),
			Mchid:       core.String(mt.Table.Config.GetWxMerchantId()),
			Description: core.String(info.Name),
			OutTradeNo:  core.String(orderid),
			Attach:      core.String(userip),
			NotifyUrl:   core.String(fmt.Sprintf("https://%s/wx/paynotify/%d", cfg.GetWxPayNotifyHost(), gameid)),
			Amount: &jsapi.Amount{
				Total: core.Int64(info.Price),
			},
			Payer: &jsapi.Payer{
				Openid: core.String(openid),
			},
		},
	)

	if err != nil {
		f5.GetSysLog().Debug("prepay err:%s", err.Error())
		return
	}
	f5.GetSysLog().Debug("prepay rsp:%s, %s", resp.String(), result.Response.Status)

	rspstr = fmt.Sprintf(wp.payhtmlstr, *resp.Appid, *resp.TimeStamp, *resp.NonceStr, *resp.Package, *resp.PaySign)
	return
}

func (wp *wxpay) VerifyPaySign(rawdata string, signaturebase64 string) error {
	// 解码签名
	signature, err := base64.StdEncoding.DecodeString(signaturebase64)
	if err != nil {
		return err
	}

	// 计算数据的SHA256哈希
	hash := sha256.New()
	hash.Write([]byte(rawdata))
	hashed := hash.Sum(nil)

	// 使用公钥验证签名
	return rsa.VerifyPKCS1v15(wp.mchpubkey, crypto.SHA256, hashed, signature)
}

func (wp *wxpay) DecryptPaydata(associatedData string, nonce string, clipherdata string) (plaindata string, err error) {
	return utils.DecryptAES256GCM(mt.Table.Config.GetWxMerchantApiKey(), associatedData, nonce, clipherdata)
}
